unit Edge;

{ =====

TEdgeForm   (c) 2001    RL Collins          rcoll@northlink.com


This form simulates the Windows-style "auto-hide" START menu tool bar.  This
form sits against an edge of the screen (left, right, top, or bottom) and will
automatically shrink itself down to a thin line when there is no activity on
the form.  If the mouse pases over it, the form autmatically expands itself
back to its full size mode.

The user may use the mouse to resize the form, or to move it to a new screen
edge, just like the Window START menu bar.

The OnResize form event is called whenever the form shrinks or expands; in
addition, OnResize is called whenever the form is moved to a new location
(Delphi does not have an OnMove event, so we use OnResize to maintain a
consistant interface).

There are two components already defined on the form: a panel component
(efPanel) and a timer (efTimer).  These need to stay as they are.  Any new
components you add to the form (buttons, labels, etc) should be placed on
top of the panel.  This way they can be safely tucked away when the form
is shrunken to a thin line along the edge of the screen.

In addtion, you may use any of the standard Delphi events for the form (or
the panel) such as OnResize, OnKeyPress, OnMouseDown, etc.

One event is already defined: OnCreate.  This event calls the initialization
for the rest of the TEdgeForm functions; however, you may insert your own
code right after the initialization routine.

It is strongly recommended that you not modify any field or routine whose
name starts with "ef" (such as 'efFindEdge', 'efDoMouseMove', etc) as these
are the heart of the TEdgeForm.  Only if you really understand what is going
on should to change them.

form properties:

AutoFull:  boolean
    Set to TRUE if the form should automatically shrink when not in use,
    or automatically expand when the mouse passes over the (shrunken) form;
    If set to FALSE, the form must be set to full-size using property 'FullSize'.
    On reading this property, returns TRUE or FALSE if the capability is
    enabled.

AutoMove:  boolean
    Set to TRUE to allow the user to re-size or move the form using the mouse
    pointer.  If set to FALSE, the form must be re-sized by setting property
    'Size', and moved to a new screen edge by setting standard form property
    'Align'.  On reading this property, returns TRUE or FALSE if the
    capability is enabled.

FullSize:  boolean
    Set to TRUE to expand the form to it's full size, or set to FALSE to
    shrink the form to a thin line along the edge of the screen.  On reading
    this property, returns TRUE if the form is currently full-size, or FALSE
    if it is a thin line.  Normally only used if property 'AutoFull' is set
    to FALSE.

Size:      integer
    Defines the full-size of the form.  If the form is aligned along the
    top or botom edge of the screen, the width of the form is the width of
    the screen, and 'Size' represents the height of the form.  If the form
    is aligned along the left or right of the screen, then the height of the
    form is the height of the screen, and 'Size' is the width of the form.

Smooth:    boolean
    Determines the transition between full-size form and the thin line along
    the edge of the screen.  Set to TRUE to enable a smooth expansion; set
    to FALSE to have the form abruptly snap to the new size.

Align:     TAlign
    Standard form alignment defined by Delphi; on the four edges 'alTop',
    'alBottom', 'alRight', and 'alLeft' are used by a TEdgeForm.  Default
    is 'alBottom'.

efTimer.Interval:   integer
    Defines the amount of time for a form to automatically shrink from
    full-size down to a thin line.  Default is 1000 msec (1 second).        

===== }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TEdgeForm = class(TForm)
    efPanel: TPanel;
    efTimer: TTimer;
    procedure FormCreate(Sender: TObject);


  private
    efVClick:       TNotifyEvent;               // saved event proc
    efVDClick:      TNotifyEvent;               // saved event proc
    efVKeyDown:     TKeyEvent;                  // saved event proc
    efVKeyPress:    TKeyPressEvent;             // saved event proc
    efVKeyUp:       TKeyEvent;                  // saved event proc
    efVMouseDown:   TMouseEvent;                // saved event proc
    efVMouseMove:   TMouseMoveEvent;            // saved event proc
    efVMouseUp:     TMouseEvent;                // saved event proc
    efVResize:      TNotifyEvent;               // saved event proc
    efVTimer:       TNotifyEvent;               // saved evnet proc


    efCursor:       TCursor;                    // default panel cursor
    efFullSize:     boolean;                    // is full or small size?
    efBigSize:      integer;                    // saved full-size width/height
    efSmallSize:    integer;                    // when it is shrunk down
    efAutoFull:     boolean;                    // automagically full-size?
    efAutoMove:     boolean;                    // use the mouse to move form?
    efSmooth:       boolean;                    // smooth or abrupt sizes
    efActive:       boolean;                    // note if mouse has moved

    efMLeft:        boolean;                    // left mouse button down?
    efSX, efSY:     integer;                    // mouse pos at mouse down
    efMX, efMY:     integer;                    // current mouse pos
    efSRect:        TRect;                      // start pos of form
    efGRect:        TRect;                      // current gray box
    efSizeEdge:     TAlign;                     // resizing the form?
    efMoveEdge:     TAlign;                     // moving the form?
    efDC:           hDC;                        // device context of desktop
    efGBox:         boolean;                    // is gray bon on/off ?


    procedure efInitialize;
    procedure efDoClick(Sender: TObject);
    procedure efDoDClick(Sender: TObject);
    procedure efDoKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure efDoKeyPress(Sender: TObject; var Key: Char);
    procedure efDoKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure efDoMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure efDoMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure efDoMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure efDoResize(Sender: TObject);
    procedure efDoTimer(Sender: TObject);


    function  efFindEdge(aX, aY, aWidth, aHeight: integer): TAlign;
    function  efGetAutoFull: boolean;
    function  efGetAutoMove: boolean;
    function  efGetFullSize: boolean;
    function  efGetSize: integer;
    function  efGetSmooth: boolean;
    procedure efMoved(var msg: TWMMove);  message WM_Move;
    procedure efMoveStart;
    procedure efMoveStop;
    procedure efMoveUpdate;
    procedure efSetAutoFull(aAuto: boolean);
    procedure efSetAutoMove(aAuto: boolean);
    procedure efSetFullSize(aFull: boolean);
    procedure efSetSize(aSize: integer);
    procedure efSetSmooth(aSmooth: boolean);
    procedure efUpdateCursor;



  public
    property AutoFull:  boolean     read efGetAutoFull  write efSetAutoFull;
    property AutoMove:  boolean     read efGetAutoMove  write efSetAutoMove;
    property FullSize:  boolean     read efGetFullSize  write efSetFullSize;
    property Size:      integer     read efGetSize      write efSetSize;
    property Smooth:    boolean     read efGetSmooth    write efSetSmooth;

  end;

var
    EdgeForm: TEdgeForm;


{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }

implementation

const
    kMouseThreshold     = 10;            // enough pixels to show we want to move

{$R *.DFM}

{ ---------------------------------------------------------------------------- }

procedure TEdgeForm.FormCreate(Sender: TObject);
begin

// =======================================
// do not change this call to efInitialize
// make all new updates after this call
// =======================================

efInitialize;

// ======================================
// any additional processing goes in here
// ======================================

end;



{ ---------------------------------------------------------------------------- }
{ initialize all EdgeForm vars and states }

procedure TEdgeForm.efInitialize;
begin

// start out by aligning to bottom: NONE or CLIENT is not used

if (not (Align in [alLeft, alRight, alTop, alBottom])) then Align := alBottom;

// save click and double-click events

if      (assigned(OnClick))         then efVClick := OnClick
else if (assigned(efPanel.OnClick)) then efVClick := efPanel.OnClick
else                                     efVClick := nil;
OnClick         := efDoClick;
efPanel.OnClick := efDoClick;


if      (assigned(OnDblClick))         then efVDClick := OnDblClick
else if (assigned(efPanel.OnDblClick)) then efVDClick := efPanel.OnDblClick
else                                        efVDClick := nil;
OnDblClick         := efDoDClick;
efPanel.OnDblClick := efDoDClick;

// save keyboard events

if (assigned(OnKeyDown))  then efVKeyDown := OnKeyDown
else                           efVKeyDown := nil;
OnKeyDown := efDoKeyDown;

if (assigned(OnKeyPress)) then efVKeyPress := OnKeyPress
else                           efVKeyPress := nil;
OnKeyPress := efDoKeyPress;

if (assigned(OnKeyUp))    then efVKeyUp := OnKeyUp
else                           efVKeyUp := nil;
OnKeyUp := efDoKeyUp;

// mouse events

if      (assigned(OnMouseDown))         then efVMouseDown := OnMouseDown
else if (assigned(efPanel.OnMouseDown)) then efVMouseDown := efPanel.OnMouseDown
else                                         efVMouseDown := nil;
OnMouseDown         := efDoMouseDown;
efPanel.OnMouseDown := efDoMouseDown;

if      (assigned(OnMouseMove))         then efVMouseMove := OnMouseMove
else if (assigned(efPanel.OnMouseMove)) then efVMouseMove := efPanel.OnMouseMove
else                                         efVMouseMove := nil;
OnMouseMove         := efDoMouseMove;
efPanel.OnMouseMove := efDoMouseMove;

if      (assigned(OnMouseUp))           then efVMouseUp := OnMouseUp
else if (assigned(efPanel.OnMouseUp))   then efVMouseUp := efPanel.OnMouseUp
else                                         efVMouseUp := nil;
OnMouseUp         := efDoMouseUp;
efPanel.OnMouseUp := efDoMouseUp;


// form or panel resizes

if      (assigned(OnResize))         then efVResize := OnResize
else if (assigned(efPanel.OnResize)) then efVResize := efPanel.OnResize
else                                      efVResize := nil;
OnResize         := efDoResize;
efPanel.OnResize := efDoResize;

// and when the timer expires

if (assigned(efTimer.OnTimer)) then efVTimer := efTimer.OnTimer
else                                efVTimer := nil;
efTimer.OnTimer := efDoTimer;

// save our default cursor for later

efCursor := efPanel.Cursor;

// start out full size

efFullSize := true;

// saved size of full form

efBigSize := efGetSize;

// when small, how much to show along the edge

efSmallSize := 3;

// automatically full-size or small size

efAutoFull := true;

// use the mouse to move or re-size the form

efAutoMove := true;

// smooth size changes

efSmooth := true;

// note if the mouse have moved lately, to kee the form full-size

efActive := true;

// mouse starting coords

efMLeft := false;
efSX    := 0;
efSY    := 0;
efMX    := 0;
efMY    := 0;
efDC    := 0;
efGBox  := false;

efSRect.Left   := 0;
efSRect.Top    := 0;
efSRect.Right  := 0;
efSRect.Bottom := 0;

efGRect.Left   := 0;
efGRect.Top    := 0;
efGRect.Right  := 0;
efGRect.Bottom := 0;

efSizeEdge := alNone;
efMoveEdge := alNone;
end;


{ ---------------------------------------------------------------------------- }
{ trap the form click event }

procedure TEdgeForm.efDoClick(Sender: TObject);
begin

// sometimes a move or resize looks like a simple click

if (efMLeft) then efMoveStop;

// form is active

efActive := true;

// if it is small, make it big

if ((efAutoFull) and (not efFullSize)) then efSetFullSize(true);

// any move or resize is definitely over

efMLeft := false;

// call user routine

if (assigned(efVClick)) then efVClick(Sender);
end;

{ ---------------------------------------------------------------------------- }
{ trap the double-click event }

procedure TEdgeForm.efDoDClick(Sender: TObject);
begin

// form is active

efActive := true;

// if it is small, make it big

if ((efAutoFull) and (not efFullSize)) then efSetFullSize(true);

// any move or resize is definitely over

efMLeft := false;

// call user routine

if (assigned(efVDClick)) then efVDClick(Sender);
end;

{ ---------------------------------------------------------------------------- }
{ trap keystroke events }

procedure TEdgeForm.efDoKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin

// form is active

efActive := true;

// if it is small, make it big

if ((efAutoFull) and (not efFullSize)) then efSetFullSize(true);

// any move or resize is definitely over

efMLeft := false;

// call user routine

if (assigned(efVKeyDown)) then efVKeyDown(Sender, Key, Shift);
end;

{ ---------------------------------------------------------------------------- }
{ trap keystroke events }

procedure TEdgeForm.efDoKeyPress(Sender: TObject; var Key: Char);
begin

// form is active

efActive := true;

// if it is small, make it big

if ((efAutoFull) and (not efFullSize)) then efSetFullSize(true);

// any move or resize is definitely over

efMLeft := false;

// call user routine

if (assigned(efVKeyPress)) then efVKeyPress(Sender, Key);
end;

{ ---------------------------------------------------------------------------- }
{ trap keystroke events }

procedure TEdgeForm.efDoKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin

// form is active

efActive := true;

// if it is small, make it big

if ((efAutoFull) and (not efFullSize)) then efSetFullSize(true);

// any move or resize is definitely over

efMLeft := false;

// call user routine

if (assigned(efVKeyUp)) then efVKeyUp(Sender, Key, Shift);
end;



{ ---------------------------------------------------------------------------- }
{ used by both the form and the panel, since the panel covers the form }
{ any activity on the form will cause full-size }
{ a left-button down marks the beginning of a move or resize operation }
{ note that anything done on a small form is ignored }

procedure TEdgeForm.efDoMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    i,n:    integer;
begin

// for is active

efActive := true;

// if small and want to be big

if ((efAutoFull) and (not efFullSize)) then
    begin
    efSetFullSize(true);
    efMLeft := false;
    end

// else if just small

else if (not efFullSize) then
    begin
    efMLeft := false;
    end

// a full-size form and a plain left-button mouse down

else if (Shift = [ssLeft]) then
    begin
    efMLeft := true;
    efSX := X;
    efSY := Y;

    efMoveStart;
    end

// everything else

else
    begin
    efMLeft := false;
    if (assigned(efVMouseDown)) then efVMouseDown(Sender, Button, Shift, X, Y);
    end;
end;


{ ---------------------------------------------------------------------------- }
{ used by both the form and the panel, since the panel covers the form }
{ any activity on the form will cause full-size }
{ note that anything done on a small form is ignored }

procedure TEdgeForm.efDoMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
    i,n:    integer;
begin

// form is active

efActive := true;

// save current mouse pos

efMX := X;
efMY := Y;

// if small and want to be big

if ((efAutoFull) and (not efFullSize)) then
    begin
    efSetFullSize(true);
    efMLeft := false;
    end

// else if just small

else if (not efFullSize) then
    begin
    efMLeft := false;
    end

// move or resize a form

else if (efMLeft) then
    begin
    efMoveUpdate;
    end

// everything else

else
    begin
    efUpdateCursor;
    efMLeft := false;
    if (assigned(efVMouseMove)) then efVMouseMove(Sender, Shift, X, Y);
    end;
end;



{ ---------------------------------------------------------------------------- }
{ used by both the form and the panel, since the panel covers the form }
{ any activity on the form will cause full-size }

procedure TEdgeForm.efDoMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

// form is active

efActive := true;

// if ending an operation

if (efMLeft) then
    begin
    efMoveStop;
    efMLeft := false;
    end

// if small and want to be big

else if ((efAutoFull) and (not efFullSize)) then
    begin
    efSetFullSize(true);
    efMLeft := false;
    end

// else if just small

else if (not efFullSize) then
    begin
    efMLeft := false;
    end

// everything else

else
    begin
    efMLeft := false;
    if (assigned(efVMouseUp)) then efVMouseUp(Sender, Button, Shift, X, Y);
    end;
end;



{ ---------------------------------------------------------------------------- }
{ trap the form resize event }

procedure TEdgeForm.efDoResize(Sender: TObject);
begin
efActive := true;
efPanel.Align := alClient;

if (assigned(efVResize)) then efVResize(Sender);
end;


{ ---------------------------------------------------------------------------- }
{ the timer keeps track of when to auto-shrink the form }

procedure TEdgeForm.efDoTimer(Sender: TObject);
var
    i,n:        integer;
    l,r,t,b:    integer;
    pp:         TPoint;
begin

// if not a full-size form, then nothing to do
// if not auto-mode, the nothing to do either

if ((not efFullSize) or (not efAutoFull)) then exit;

// if in the middle of resize or move

if (efMLeft) then exit;

// if already active, then do nothing here
// otherwise, see if the mouse is sitting over the form

if (not efActive) then
    begin
    l := Left;
    r := l + Width;
    t := Top;
    b := t + Height;
    GetCursorPos(pp);
    if ((l <= pp.X) and (pp.X <= r) and (t <= pp.Y) and (pp.Y <= b)) then efActive := true;
    end;

// if not active, then shrink the form

if (not efActive) then efSetFullSize(false);

// reset to test for activity

efActive := false;

// call user routine

if (assigned(efVTimer)) then efVTimer(Sender);
end;


{ ---------------------------------------------------------------------------- }
{ find the form or screen edge the coords are closest to }

function TEdgeForm.efFindEdge(aX, aY, aWidth, aHeight: integer): TAlign;
var
    i,n:        integer;
    l,r,t,b:    integer;
    ee:         TAlign;
begin

// an outrageous starting point

ee := alNone;
n  := 5000;

// distance to left, right, top, bottom edges

l := abs(aX);
r := abs(aWidth - aX);
t := abs(aY);
b := abs(aHeight - aY);

// find closest one

if (l < n) then
    begin
    ee := alLeft;
    n  := l;
    end;

if (r < n) then
    begin
    ee := alRight;
    n  := r;
    end;

if (t < n) then
    begin
    ee := alTop;
    n  := t;
    end;

if (b < n) then
    begin
    ee := alBottom;
    n  := b;
    end;

// must be within threshold value

if (n > kMouseThreshold) then ee := alNone;

// done

efFindEdge := ee;
end;

{ ---------------------------------------------------------------------------- }

function TEdgeForm.efGetAutoFull: boolean;
begin
efGetAutoFull := efAutoFull;
end;

{ ---------------------------------------------------------------------------- }

function TEdgeForm.efGetAutoMove: boolean;
begin
efGetAutoMove := efAutoMove;
end;

{ ---------------------------------------------------------------------------- }
{ set full-size mode }

function TEdgeForm.efGetFullSize: boolean;
begin
efGetFullSize := efFullSize;
end;

{ ---------------------------------------------------------------------------- }
{ get the current size of the edge form -- width or height }

function TEdgeForm.efGetSize: integer;
var
    i,n:    integer;
    cs,ns:  integer;
begin
if ((Align = alLeft) or (Align = alRight)) then cs := Width
else                                            cs := Height;

efGetSize := cs;
end;

{ ---------------------------------------------------------------------------- }

function TEdgeForm.efGetSmooth: boolean;
begin
efGetSmooth := efSmooth;
end;


{ ---------------------------------------------------------------------------- }
{ this is called by Windows when the user moves the form on the screen }
{ surprisingly, this is not a Delphi event; e.g., OnMove }

procedure TEdgeForm.efMoved(var msg: TWMMove);
begin
efActive := true;
efPanel.Align := alClient;

// when moved, tell the user that something has changed

if (assigned(efVResize)) then efVResize(Sender);
end;

{ ---------------------------------------------------------------------------- }
{ here to start a mouse move or resize }

procedure TEdgeForm.efMoveStart;
var
    i,n:    integer;
    aa:     TAlign;
    ee:     TAlign;
begin

// assume we started

efMLeft    := true;
efSizeEdge := alNone;
efMoveEdge := alNone;

// quick ref

aa := Align;

// mouse near which edge?

ee := efFindEdge(efSX, efSY, Width, Height);

// start a resize?

if      ((aa = alLeft)   and (ee = alRight))  then efSizeEdge := alRight
else if ((aa = alRight)  and (ee = alLeft))   then efSizeEdge := alLeft
else if ((aa = alTop)    and (ee = alBottom)) then efSizeEdge := alBottom
else if ((aa = alBottom) and (ee = alTop))    then efSizeEdge := alTop

// or a move edge?

else if ((aa = alLeft)   and (ee = alTop))    then efSizeEdge := alClient
else if ((aa = alLeft)   and (ee = alBottom)) then efSizeEdge := alClient
else if ((aa = alRight)  and (ee = alTop))    then efSizeEdge := alClient
else if ((aa = alRight)  and (ee = alBottom)) then efSizeEdge := alClient
else if ((aa = alTop)    and (ee = alLeft))   then efSizeEdge := alClient
else if ((aa = alTop)    and (ee = alRight))  then efSizeEdge := alClient
else if ((aa = alBottom) and (ee = alLeft))   then efSizeEdge := alClient
else if ((aa = alBottom) and (ee = alRight))  then efSizeEdge := alClient

// maybe we didn't start after all

else efMLeft := false;

// save device context of desktop to draw gray lines

if (efMLeft) then
    begin
    efDC   := GetDC(0);
    efGBox := false;

    efSRect.Left   := Left;
    efSRect.Top    := Top;
    efSRect.Right  := Left + Width;
    efSRect.Bottom := Top + Height;

    efGRect.Left   := Left;
    efGRect.Top    := Top;
    efGRect.Right  := Left + Width;
    efGRect.Bottom := Top + Height;
    end;
end;

{ ---------------------------------------------------------------------------- }
{ here to finish a mouse move or reseize }

procedure TEdgeForm.efMoveStop;
var
    i,n:    integer;
    l,t:    integer;
    w,h:    integer;
begin

// if we are not doing anything ...

if (not efMLeft) then exit;

// turn off any left-over gray box

if (efGBox) then
    begin
    DrawFocusRect(efDC, efGRect);
    efGBox := false;
    end;

// done with desktop graphics

ReleaseDC(0, efDC);
efDC := 0;

// save state

l := efGRect.Left;
t := efGRect.Top;
w := efGRect.Right  - efGRect.Left;
h := efGRect.Bottom - efGRect.Top;

// update size parameters

if      (efSizeEdge = alLeft)   then efBigSize := w
else if (efSizeEdge = alRight)  then efBigSize := w
else if (efSizeEdge = alTop)    then efBigSize := h
else if (efSizeEdge = alBottom) then efBigSize := h
else if (efMoveEdge <> alNone)  then Align     := efMoveEdge;

// update window

SetBounds(l, t, w, h);
efPanel.Align := alClient;
efMLeft := false;
end;

{ ---------------------------------------------------------------------------- }
{ here to update a mouse move or resize }

procedure TEdgeForm.efMoveUpdate;
var
    i,n:    integer;
    dx,dy:  integer;
    ee:     TAlign;
begin

// if not updating, do nothing

if (not efMLeft) then exit;

// turn off any gray box that is displayed

if (efGBox) then
    begin
    DrawFocusRect(efDC, efGRect);
    efGBox := false;
    end;

// search for a move edge, or a delta size

if (efSizeEdge = alClient) then
    begin
    dx := efMX + Left;
    dy := efMY + Top;
    efMoveEdge := efFindEdge(efMX, efMY, Screen.Width, Screen.Height);
    end
else
    begin
    dx := efMX - efSX;
    dy := efMY - efSY;
    end;

// if changing size ...

if (efSizeEdge = alLeft) then
    begin
    efGRect.Left := efSRect.Left + dx;
    end
else if (efSizeEdge = alRight) then
    begin
    efGRect.Right := efSRect.Right + dx;
    end
else if (efSizeEdge = alTop) then
    begin
    efGRect.Top := efSRect.Top + dy;
    end
else if (efSizeEdge = alBottom) then
    begin
    efGRect.Bottom := efSRect.Bottom + dy;
    end

// move to a new edge?

else if (efMoveEdge = alLeft) then
    begin
    efGRect.Left   := 0;
    efGRect.Top    := 0;
    efGRect.Right  := efBigSize;
    efGRect.Bottom := Screen.Height;
    end
else if (efMoveEdge = alRight) then
    begin
    efGRect.Left   := Screen.Width - efBigSize;
    efGRect.Top    := 0;
    efGRect.Right  := Screen.Width;
    efGRect.Bottom := Screen.Height;
    end
else if (efMoveEdge = alTop) then
    begin
    efGRect.Left   := 0;
    efGRect.Top    := 0;
    efGRect.Right  := Screen.Width;
    efGRect.Bottom := efBigSize;
    end
else if (efMoveEdge = alBottom) then
    begin
    efGRect.Left   := 0;
    efGRect.Top    := Screen.Height - efBigSize;
    efGRect.Right  := Screen.Width;
    efGRect.Bottom := Screen.Height;
    end

// nothing happening

else
    begin
    efGRect.Left   := efSRect.Left;
    efGRect.Top    := efSRect.Top;
    efGRect.Right  := efSRect.Right;
    efGRect.Bottom := efSRect.Bottom;
    end;

// turn on gray box display

if (not efGBox) then
    begin
    DrawFocusRect(efDC, efGRect);
    efGBox := true;
    end;
end;


{ ---------------------------------------------------------------------------- }

procedure TEdgeForm.efSetAutoFull(aAuto: boolean);
begin
efAutoFull := aAuto;
end;

{ ---------------------------------------------------------------------------- }

procedure TEdgeForm.efSetAutoMove(aAuto: boolean);
begin
efAutoMove := aAuto;
end;


{ ---------------------------------------------------------------------------- }
{ set full size state }

procedure TEdgeForm.efSetFullSize(aFull: boolean);
begin

// save new state

efFullSize := aFull;

// make a full-size form?
// setting the panel visible/not-visible will show or hide the controls on it

if (efFullSize) then
    begin
    efSetSize(efBigSize);
    efPanel.Visible := true;
    efPanel.Align := alClient;
    end

// or a shrunken bar along the edge?

else
    begin
    efPanel.Visible := false;
    efSetSize(efSmallSize);
    end;

// update cursors

efUpdateCursor;
end;

{ ---------------------------------------------------------------------------- }
{ set new size of edge form }

procedure TEdgeForm.efSetSize(aSize: integer);
var
    i,n:    integer;
    cs,ns:  integer;
begin

// get current size

if ((Align = alLeft) or (Align = alRight)) then cs := Width
else                                            cs := Height;

// and the new size

ns := aSize;
if (ns < 0) then ns := 0;

// smooth transition?

if (efSmooth) then
    begin
    while (cs < ns) do
        begin
        cs := cs + 1;
        if ((Align = alLeft) or (Align = alRight)) then Width  := cs
        else                                            Height := cs;
        Application.ProcessMessages;
        end;
    while (cs > ns) do
        begin
        cs := cs - 1;
        if ((Align = alLeft) or (Align = alRight)) then Width  := cs
        else                                            Height := cs;
        Application.ProcessMessages;
        end;
    end

// no, abrupt change

else
    begin
    if ((Align = alLeft) or (Align = alRight)) then Width  := ns
    else                                            Height := ns;
    end;

// make sure the panel keeps up with the form size

efPanel.Align := alClient;
end;

{ ---------------------------------------------------------------------------- }

procedure TEdgeForm.efSetSmooth(aSmooth: boolean);
begin
efSmooth := aSmooth;
end;

{ ---------------------------------------------------------------------------- }
{ update cursor style depending upon mouse position }

procedure TEdgeForm.efUpdateCursor;
var
    i,n:    integer;
    cc:     TCursor;
    aa:     TAlign;
    ee:     TAlign;
begin

// if a small form, then just the regular old cursor

if (not efFullSize) then
    begin
    cc := crDefault;
    end

// we do not move or resize the form

else if (not efAutoMove) then
    begin
    cc := efCursor;
    end

// check if near an edge

else
    begin
    aa := Align;
    ee := efFindEdge(efMX, efMY, Width, Height);

// else on the re-size edge?

    if      ((aa = alLeft)   and (ee = alRight))  then cc := crSizeWE
    else if ((aa = alRight)  and (ee = alLeft))   then cc := crSizeWE
    else if ((aa = alTop)    and (ee = alBottom)) then cc := crSizeNS
    else if ((aa = alBottom) and (ee = alTop))    then cc := crSizeNS

// or a move edge?

    else if ((aa = alLeft)   and (ee = alTop))    then cc := crDrag
    else if ((aa = alLeft)   and (ee = alBottom)) then cc := crDrag
    else if ((aa = alRight)  and (ee = alTop))    then cc := crDrag
    else if ((aa = alRight)  and (ee = alBottom)) then cc := crDrag
    else if ((aa = alTop)    and (ee = alLeft))   then cc := crDrag
    else if ((aa = alTop)    and (ee = alRight))  then cc := crDrag
    else if ((aa = alBottom) and (ee = alLeft))   then cc := crDrag
    else if ((aa = alBottom) and (ee = alRight))  then cc := crDrag

// just plain old cursor

    else cc := efCursor;
    end;

// the cursor is used for both the form and the panel

Cursor := cc;
efPanel.Cursor := cc;
end;

{ ---------------------------------------------------------------------------- }
{ ============================================================================ }
{ all new control definitions and user procedures go after this lise }
{ ============================================================================ }
{ ---------------------------------------------------------------------------- }





end.
